const CosmosClient = require('@azure/cosmos').CosmosClient;
const BaseDriver = require('@cubejs-backend/query-orchestrator/driver/BaseDriver');

class CosmosDbDriver extends BaseDriver {
  constructor(config) {
    super();
    this.config = {
      connectionString: process.env.CUBEJS_COSMOS_CONNECTION_STRING,
      databaseId: process.env.CUBEJS_COSMOS_DATABASE_ID,
      containerId: process.env.CUBEJS_COSMOS_CONTAINER_ID,
      useEmulator: process.env.CUBEJS_COSMOS_USE_EMULATOR,
      ...config
    };
    this.client = new CosmosClient(this.config.connectionString);
    this.databaseId = this.config.databaseId;
    this.containerId = this.config.containerId;
  }

  async testConnection() {
    return this.query('SELECT TOP 1 * FROM c');
  }

  sleep(ms) {
    return new Promise((resolve) => {
      setTimeout(() => resolve(), ms);
    });
  }

  async createDatabaseIfNotExists() {
    await this.client.databases.createIfNotExists({ id: this.databaseId });
  }

  async createContainerIfNotExists() {
    await this.client.database(this.databaseId).containers.createIfNotExists({ id: this.containerId });
  }

  async createItemIfNotExists(document) {
    await this.client.database(this.databaseId).container(this.containerId).items.create(document);
  }

  async deleteContainer() {
    await this.client.database(this.databaseId).container(this.containerId).delete();
  }

  async query(query, parameters) {
    const querySpec = {
      query: query,
      parameters: parameters
    }
    console.log(`Querying ${this.databaseId} and ${this.containerId}`);
    const { resources: results } = await this.client
      .database(this.databaseId).container(this.containerId).items.query(querySpec).fetchAll();
    return results;
  }
}

module.exports = CosmosDbDriver;
